@@ -19,9 +19,13 @@
 #'         \code{tbl} class attributes for data frames. Some known issues with \code{\link[haven]{labelled}}
 #'         class vectors have already been fixed, so it might be that this function
 #'         will become redundant in the future. Currently, data frames with \code{tbl_df} and
-#'         \code{tbl} class attributes may cause difficulties when indexing columns
-#'         like \code{data.frame[, colnr]} - only \code{data.frame[[colnr]]} seems
-#'         to be safe when accessing data frame columns from within function calls.
+#'         \code{tbl} class attributes may cause difficulties when indexing columns,
+#'         because the \pkg{haven} package's \code{read_*}-functions return a \code{tibble},
+#'         which never simplifies data frame when indexing like \code{data.frame[, colnr]}.
+#'         Thus, packages or functions that expect \code{data.frame[, colnr]} to return a
+#'         vector, might throw an error - only \code{data.frame[[colnr]]} is safe
+#'         when accessing data frame columns from within function calls (see
+#'         \code{\link[tibble]{tibble-package}} for details).
 #'         \cr \cr
 #'         Data frames will be converted into labelled data frames (see \code{\link{lbl_df}}).
 #'
@@ -30,16 +34,16 @@
 unlabel <- function(x) {
   # check if complete data frame or only single
   # vector should be converted
-  if (is.data.frame(x) || is.matrix(x)) {
+  if (is.data.frame(x)) {
     # create progress bar
     pb <- utils::txtProgressBar(min = 0,
                                 max = ncol(x),
                                 style = 3)
     # tell user...
     message("Converting labelled-classes. Please wait...\n")
-    for (i in 1:ncol(x)) {
+    for (i in seq_len(ncol(x))) {
       # remove labelled class
-      if (is_labelled(x[[i]])) x[[i]] <- unclass(x[[i]])
+      if (haven::is.labelled(x[[i]])) x[[i]] <- unclass(x[[i]])
       # update progress bar
       utils::setTxtProgressBar(pb, i)
     }
@@ -48,7 +52,7 @@ unlabel <- function(x) {
     class(x) <- c("lbl_df", "data.frame")
   } else {
     # remove labelled class
-    if (is_labelled(x)) x <- unclass(x)
+    if (haven::is.labelled(x)) x <- unclass(x)
   }
   return(x)
 }
